/*
 * xhtmlwidget.c
 * xhtmlite
 *
 * by Doug Wolfe
 * doug@wolfescience.com 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc.,  59 Temple Place - Suite 330, Cambridge, MA 02139, USA.
 *
 */

#include <gtk/gtk.h>
 
// Find a child widget by name, starting from an enclosing container

static gboolean
xhtmlite_widget_name (GtkWidget * widget, gchar * name)
{
  if (strcmp (gtk_widget_get_name (widget), name) == 0) {
    return TRUE;
  }
  else return FALSE;
}

// parent must be a container: loop through its children, finding a widget with the name 'name'

GtkWidget *
xhtmlite_widget_find_child_by_name (GtkWidget *parent, gchar *name)
{
  GtkWidget *widget, *found_widget = NULL;
  GList *children, *child_l;

  if (xhtmlite_widget_name (parent, name))
    found_widget = parent;

  else {
    if (GTK_CONTAINER(parent)) {
      children = gtk_container_children (GTK_CONTAINER(parent));
      for (child_l=children; child_l; child_l=child_l->next) {
        if (GTK_IS_WIDGET(child_l->data)) {
          widget = GTK_WIDGET (child_l->data);
          if (xhtmlite_widget_name (widget, name)) {
            found_widget = widget;
            break;
          }
          if (GTK_IS_CONTAINER (widget)) {
            found_widget = xhtmlite_widget_find_child_by_name (widget, name);
            if (found_widget != NULL)
              break;
          }
        }
      }
    }
  }
  return found_widget;
}

GtkWidget *
xhtmlite_widget_find_parent (GtkWidget * child)
{
  GtkWidget * parent = gtk_widget_get_parent(child);
  if(parent != NULL && GTK_IS_CONTAINER(parent))
  {
     return xhtmlite_widget_find_parent(parent);
  }
  else
  {
      return child;
  }
}

GtkWidget *
xhtmlite_widget_find_child_by_name_active (GtkWidget *parent, gchar *name)
{
  GtkWidget *widget, *found_widget = NULL;
  GList *children, *child_l;

  if (xhtmlite_widget_name(parent, name) && gtk_widget_is_focus(found_widget))
    found_widget = parent;

  else {
    if (GTK_CONTAINER(parent)) {
      children = gtk_container_children (GTK_CONTAINER(parent));
      for (child_l=children; child_l; child_l=child_l->next) {
        if (GTK_IS_WIDGET(child_l->data)) {
          widget = GTK_WIDGET (child_l->data);
          if (xhtmlite_widget_name(widget, name) && gtk_widget_is_focus(found_widget)) {
            found_widget = widget;
            break;
          }
          if (GTK_IS_CONTAINER (widget)) {
            found_widget = xhtmlite_widget_find_child_by_name (widget, name);
            if (found_widget != NULL)
              break;
          }
        }
      }
    }
  }
  return found_widget;
}

GtkWidget*
xhtmlite_widget_find_parent_window (GtkWidget *widget)
{
  GtkWidget *parent;

  for (;;)
    {
      if (GTK_IS_MENU (widget))
        parent = gtk_menu_get_attach_widget (GTK_MENU (widget));
      else
        parent = widget->parent;
      if (!parent)
        parent = (GtkWidget*) g_object_get_data (G_OBJECT (widget), "GladeParentKey");
      if (parent == NULL)
        break;
      widget = parent;
    }

  return parent;
}

gint
g_str_get_length (const gchar *str)
{
    if (str == NULL) return 0;
    else return g_utf8_strlen (str, -1);
}

